@@ -3,7 +3,7 @@
 use super::{
     super::{
         base::{credential::Credential, download::RangeReaderBuilder as BaseRangeReaderBuilder},
-        config::{build_async_range_reader_builder_from_config, Config, Timeouts},
+        config::{build_range_reader_builder_from_config, Config, Timeouts},
     },
     dot::{ApiName, DotType, Dotter},
     host_selector::{HostInfo, HostSelector, HostSelectorBuilder},
@@ -87,7 +87,7 @@ pub fn sign_download_url_with_lifetime(
 }
 
 #[derive(Debug)]
-pub(crate) struct AsyncRangeReaderBuilder(BaseRangeReaderBuilder);
+pub(super) struct AsyncRangeReaderBuilder(BaseRangeReaderBuilder);
 
 impl From<BaseRangeReaderBuilder> for AsyncRangeReaderBuilder {
     fn from(builder: BaseRangeReaderBuilder) -> Self {
@@ -102,11 +102,11 @@ impl From<AsyncRangeReaderBuilder> for BaseRangeReaderBuilder {
 }
 
 impl AsyncRangeReaderBuilder {
-    pub(crate) fn take_key(&mut self) -> String {
+    pub(super) fn take_key(&mut self) -> String {
         take(&mut self.0.key)
     }
 
-    pub(crate) fn build(self) -> AsyncRangeReader {
+    pub(super) fn build(self) -> AsyncRangeReader {
         AsyncRangeReader(Arc::new(AsyncLazy::new(Box::pin(async move {
             self.build_inner().await
         }))))
@@ -243,12 +243,12 @@ impl AsyncRangeReaderBuilder {
     }
 
     pub(crate) fn from_config(key: String, config: &Config) -> Self {
-        build_async_range_reader_builder_from_config(key, config)
+        build_range_reader_builder_from_config(key, config).into()
     }
 }
 
 #[derive(Clone)]
-pub(crate) struct AsyncRangeReader(Arc<AsyncLazy<Arc<AsyncRangeReaderInner>>>);
+pub(super) struct AsyncRangeReader(Arc<AsyncLazy<Arc<AsyncRangeReaderInner>>>);
 
 #[derive(Debug)]
 struct AsyncRangeReaderInner {
@@ -264,42 +264,19 @@ struct AsyncRangeReaderInner {
 }
 
 impl AsyncRangeReader {
-    // pub(super) fn from_env_with_extra_items(
-    //     key: String,
-    // ) -> Option<(Self, String, Option<usize>, Option<usize>)> {
-    //     with_current_qiniu_config(|config| {
-    //         config.and_then(|config| {
-    //             config.with_key(&key.to_owned(), |config| {
-    //                 let config = Arc::new(config.to_owned());
-    //                 let range_reader_config = config.to_owned();
-    //                 (
-    //                     config.max_retry_concurrency(),
-    //                     config.retry(),
-    //                     Box::pin(async move {
-    //                         config
-    //                             .get_or_init_async_range_reader_inner(move || async move {
-    //                                 AsyncRangeReaderBuilder::from_config(
-    //                                     String::new(),
-    //                                     &range_reader_config,
-    //                                 )
-    //                                 .build_inner()
-    //                                 .await
-    //                             })
-    //                             .await
-    //                     }),
-    //                 )
-    //             })
-    //         })
-    //     })
-    //     .map(|(max_retry_concurrency, total_tries, fut)| {
-    //         (
-    //             Self(Arc::new(AsyncLazy::new(fut))),
-    //             key,
-    //             max_retry_concurrency,
-    //             total_tries,
-    //         )
-    //     })
-    // }
+    pub(super) async fn dot(
+        &self,
+        dot_type: DotType,
+        api_name: ApiName,
+        successful: bool,
+        elapsed_duration: Duration,
+    ) -> IoResult<()> {
+        let inner = self.0.get().await;
+        inner
+            .dotter
+            .dot(dot_type, api_name, successful, elapsed_duration)
+            .await
+    }
 
     pub(super) async fn update_urls(&self) -> bool {
         self.inner().await.io_selector.update_hosts().await
@@ -342,7 +319,7 @@ impl AsyncRangeReader {
         pos: u64,
         size: u64,
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         on_host_selected: F,
@@ -353,7 +330,6 @@ impl AsyncRangeReader {
         return self.with_retries(
             key,
             Method::GET,
-            ApiName::RangeReaderReadAt,
             async_task_id,
             tries_info,
             trying_hosts,
@@ -422,7 +398,7 @@ impl AsyncRangeReader {
         &self,
         ranges: &[(u64, u64)],
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         on_host_selected: F,
@@ -431,7 +407,6 @@ impl AsyncRangeReader {
             .with_retries(
                 key,
                 Method::GET,
-                ApiName::RangeReaderReadAt,
                 async_task_id,
                 tries_info,
                 trying_hosts,
@@ -554,7 +529,7 @@ impl AsyncRangeReader {
     pub(super) async fn exist<F: FnMut(HostInfo) -> Fut, Fut: Future<Output = ()>>(
         &self,
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         on_host_selected: F,
@@ -562,7 +537,6 @@ impl AsyncRangeReader {
         self.with_retries(
             key,
             Method::HEAD,
-            ApiName::RangeReaderExist,
             async_task_id,
             tries_info,
             trying_hosts,
@@ -614,7 +588,7 @@ impl AsyncRangeReader {
     pub(super) async fn file_size<F: FnMut(HostInfo) -> Fut, Fut: Future<Output = ()>>(
         &self,
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         on_host_selected: F,
@@ -622,7 +596,6 @@ impl AsyncRangeReader {
         self.with_retries(
             key,
             Method::HEAD,
-            ApiName::RangeReaderFileSize,
             async_task_id,
             tries_info,
             trying_hosts,
@@ -676,7 +649,7 @@ impl AsyncRangeReader {
     pub(super) async fn download<F: FnMut(HostInfo) -> Fut, Fut: Future<Output = ()>>(
         &self,
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         mut on_host_selected: F,
@@ -716,7 +689,7 @@ impl AsyncRangeReader {
     async fn _download<F: FnMut(HostInfo) -> Fut, Fut: Future<Output = ()>>(
         &self,
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         init_from: u64,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
@@ -728,7 +701,6 @@ impl AsyncRangeReader {
             .with_retries(
                 key,
                 Method::GET,
-                ApiName::RangeReaderDownloadTo,
                 async_task_id,
                 tries_info,
                 trying_hosts,
@@ -818,7 +790,7 @@ impl AsyncRangeReader {
         &self,
         size: u64,
         key: &str,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         on_host_selected: F,
@@ -826,7 +798,6 @@ impl AsyncRangeReader {
         return self.with_retries(
             key,
             Method::GET,
-            ApiName::RangeReaderReadLastBytes,
             async_task_id,
             tries_info,
             trying_hosts,
@@ -896,15 +867,13 @@ impl AsyncRangeReader {
         &self,
         key: &str,
         method: Method,
-        api_name: ApiName,
-        async_task_id: usize,
+        async_task_id: u32,
         tries_info: TriesInfo<'_>,
         trying_hosts: &TryingHosts,
         mut on_host_selected: F2,
         mut for_each_url: F,
     ) -> IoResult3<T> {
         let begin_at = SystemTime::now();
-        let begin_at_instant = Instant::now();
         let mut last_error: Option<IoError> = None;
         let inner = self.inner().await;
 
@@ -913,7 +882,6 @@ impl AsyncRangeReader {
             if tries >= tries_info.total_tries {
                 return IoResult3::NoMoreTries(last_error);
             }
-            let last_try = tries_info.total_tries - tries <= 1;
 
             let chosen_io_info = {
                 let mut guard = trying_hosts.lock().await;
@@ -963,11 +931,6 @@ impl AsyncRangeReader {
             {
                 Ok(result) => {
                     inner.io_selector.reward(chosen_io_info.host()).await;
-                    inner
-                        .dotter
-                        .dot(DotType::Sdk, api_name, true, begin_at_instant.elapsed())
-                        .await
-                        .ok();
                     inner
                         .dotter
                         .dot(
@@ -995,15 +958,10 @@ impl AsyncRangeReader {
                         )
                         .await
                         .ok();
-                    if !punished || last_try {
-                        inner
-                            .dotter
-                            .dot(DotType::Sdk, api_name, false, begin_at_instant.elapsed())
-                            .await
-                            .ok();
-                        return Err(err).into();
-                    } else {
+                    if punished {
                         last_error = Some(err);
+                    } else {
+                        return Err(err).into();
                     }
                 }
             }
@@ -1440,13 +1398,6 @@ mod tests {
                 assert_eq!(record.success_count(), Some(2));
                 assert_eq!(record.failed_count(), Some(0));
             }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(DotType::Sdk, ApiName::RangeReaderReadAt))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(2));
-                assert_eq!(record.failed_count(), Some(0));
-            }
         });
         Ok(())
     }
@@ -1499,7 +1450,7 @@ mod tests {
                 )
                 .await
             {
-                Result3::Err(..) => {}
+                Result3::NoMoreTries(..) => {}
                 _ => unreachable!(),
             }
             assert_eq!(io_called.load(Relaxed), 3);
@@ -1512,13 +1463,6 @@ mod tests {
                 assert_eq!(record.success_count(), Some(0));
                 assert_eq!(record.failed_count(), Some(3));
             }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(DotType::Sdk, ApiName::RangeReaderReadAt))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(0));
-                assert_eq!(record.failed_count(), Some(1));
-            }
         });
         Ok(())
     }
@@ -1584,13 +1528,6 @@ mod tests {
                 assert_eq!(record.success_count(), Some(0));
                 assert_eq!(record.failed_count(), Some(1));
             }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(DotType::Sdk, ApiName::RangeReaderReadAt))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(0));
-                assert_eq!(record.failed_count(), Some(1));
-            }
         });
         Ok(())
     }
@@ -1657,16 +1594,6 @@ mod tests {
                 assert_eq!(record.success_count(), Some(1));
                 assert_eq!(record.failed_count(), Some(0));
             }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(
-                        DotType::Sdk,
-                        ApiName::RangeReaderReadLastBytes,
-                    ))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(1));
-                assert_eq!(record.failed_count(), Some(0));
-            }
         });
         Ok(())
     }
@@ -1753,33 +1680,6 @@ mod tests {
                 assert_eq!(record.success_count(), Some(3));
                 assert_eq!(record.failed_count(), Some(0));
             }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(DotType::Sdk, ApiName::RangeReaderExist))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(1));
-                assert_eq!(record.failed_count(), Some(0));
-            }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(
-                        DotType::Sdk,
-                        ApiName::RangeReaderFileSize,
-                    ))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(1));
-                assert_eq!(record.failed_count(), Some(0));
-            }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(
-                        DotType::Sdk,
-                        ApiName::RangeReaderDownloadTo,
-                    ))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(1));
-                assert_eq!(record.failed_count(), Some(0));
-            }
         });
         Ok(())
     }
@@ -1828,7 +1728,7 @@ mod tests {
                 )
                 .await
             {
-                Result3::Err(_) => {}
+                Result3::NoMoreTries(_) => {}
                 _ => unreachable!(),
             }
 
@@ -1843,7 +1743,7 @@ mod tests {
                 )
                 .await
             {
-                Result3::Err(_) => {}
+                Result3::NoMoreTries(_) => {}
                 _ => unreachable!(),
             }
 
@@ -1858,40 +1758,13 @@ mod tests {
                 )
                 .await
             {
-                Result3::Err(_) => {}
+                Result3::NoMoreTries(_) => {}
                 _ => unreachable!(),
             }
 
             assert_eq!(counter.load(Relaxed), 3 * 3);
 
             sleep(Duration::from_secs(5)).await;
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(
-                        DotType::Sdk,
-                        ApiName::RangeReaderFileSize,
-                    ))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(0));
-                assert_eq!(record.failed_count(), Some(1));
-            }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(DotType::Sdk, ApiName::RangeReaderExist))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(0));
-                assert_eq!(record.failed_count(), Some(1));
-            }
-            {
-                let record = records_map
-                    .get(&DotRecordKey::new(
-                        DotType::Sdk,
-                        ApiName::RangeReaderDownloadTo,
-                    ))
-                    .unwrap();
-                assert_eq!(record.success_count(), Some(0));
-                assert_eq!(record.failed_count(), Some(1));
-            }
             {
                 let record = records_map
                     .get(&DotRecordKey::new(DotType::Http, ApiName::IoGetfile))
@@ -2244,7 +2117,7 @@ mod tests {
                     )
                     .await
                 {
-                    Result3::Err(..) => {}
+                    Result3::NoMoreTries(..) => {}
                     _ => unreachable!(),
                 }
                 assert_eq!(c.load(Relaxed), 3);
@@ -2278,7 +2151,7 @@ mod tests {
                     )
                     .await
                 {
-                    Result3::Err(..) => {}
+                    Result3::NoMoreTries(..) => {}
                     _ => unreachable!(),
                 }
                 assert_eq!(c.load(Relaxed), 6);
