@@ -263,9 +263,11 @@ impl HostPunisher {
         hosts_count * usize::from(self.max_punished_hosts_percent) / 100
     }
 
-    fn is_available(&self, punished_info: &PunishedInfo) -> bool {
-        !punished_info.failed_to_connect
-            && punished_info.continuous_punished_times <= self.max_punished_times
+    fn is_available(&self, punished_info: &PunishedInfo, connection_sensitive: bool) -> bool {
+        if connection_sensitive && punished_info.failed_to_connect {
+            return false;
+        }
+        punished_info.continuous_punished_times <= self.max_punished_times
     }
 
     fn is_punishment_expired(&self, punished_info: &PunishedInfo) -> bool {
@@ -432,7 +434,7 @@ impl HostSelector {
                     .get(host)
                     .map(|punished_info| {
                         self.host_punisher.is_punishment_expired(&punished_info)
-                            || self.host_punisher.is_available(&punished_info)
+                            || self.host_punisher.is_available(&punished_info, true)
                     })
                     .unwrap_or(true)
             })
@@ -584,7 +586,7 @@ impl HostSelector {
                     host, punished_info.continuous_punished_times, punished_info.timeout_power
                 );
 
-                if !self.host_punisher.is_available(&punished_info) {
+                if !self.host_punisher.is_available(&punished_info, false) {
                     return PunishResult::PunishedAndFreezed;
                 }
             }
@@ -618,7 +620,7 @@ impl HostSelector {
     }
 
     fn is_satisfied_with(&self, punished_info: &PunishedInfo) -> bool {
-        self.host_punisher.is_available(punished_info)
+        self.host_punisher.is_available(punished_info, true)
             && self.hosts_updater.current_timeout_power.load(Relaxed) >= punished_info.timeout_power
     }
 }
